
private struct Bucket<Key : Hashable, Value> : CustomDebugStringConvertible, SequenceType {
    var _contents:[(Key, Value)] = []
    
    func indexOfKey(key:Key) -> Int? {
        return _contents.indexOf { (keyToCheck, _) in key == keyToCheck } //yup, a linear search of the array of pairs. Hash tables are O(n) when many things end up in the same bucket. This can cause a problem in cases where someone malicious can control the keys being set in a hash table. If they can force many collisions, they can cause unexpected massive slowdowns. This can be avoided using, cuckoo hashing (see comment at the bottom), trees, or various other approaches.
    }
    
    func get(key: Key) -> Value? {
        guard let index = indexOfKey(key) else { return nil }
        return _contents[index].1
    }
    
    mutating func set(value:Value?, forKey key:Key) {
        if let index = indexOfKey(key) { //if indexOfKey() returns non-nil, then we already have a value for this key in this bucket
            if let value = value {
                _contents[index].1 = value //replace the value for this key
            } else {
                _contents.removeAtIndex(index) //I'm always a little dubious about the "set nil to remove" thing, but it's consistent, and it makes things a little simpler for us here
            }
        } else if let value = value {
            _contents.append((key, value)) //not a replacement, and not a removal, so this is either a collision or the initial insert of a value. Either way, we just stick the new pair in
        }
    }
    
    func generate() -> IndexingGenerator<[(Key, Value)]> {
        return _contents.generate()
    }
    
    var count : Int {
        return _contents.count
    }
    
    var debugDescription : String {
        return "(" + _contents.map { (key, value) in "\(key) : \(value)" }.joinWithSeparator(", ") + ")"
    }
}

// I'm intentionally leaving out CollectionType-conformance, and other niceties in the interests of keeping this focused on "what is a hash table and how does it do its thing". I put a SequenceType conformance in because "why is iterating a Dictionary not in order?" is such a common question on StackOverflow, so it's neat to see how that iteration could be implemented.
public struct MyDictionary<Key : Hashable, Value> : DictionaryLiteralConvertible, CustomDebugStringConvertible, SequenceType {
    private var _buckets:[Bucket<Key, Value>]
    
    private func bucketIndexForKey(key:Key) -> Int {
        /*
            The only requirement for .hashValue is that if two keys are equal, they must have the same hash value.
            Ideally though (i.e. if it's a good implementation of hashValue), when the keys are different, the hash values will also be as different as possible. This lets us return a roughly even distribution of numbers from this function
            Weirdly, hashValue can be negative in Swift. abs() isn't a great way to handle that, since it loses 1 bit of the hash, but it works.
        */
        return abs(key.hashValue) % (_buckets.count) //modular arithmetic "wraps around", so we can use it to turn an arbitrarily large hash value into a valid index into our array
    }
    
    private var capacity : Int {
        return _buckets.count
    }
    
    //The load factor represents how "full" the hash table is. As it gets more full, the chance of more than one value per bucket (a "collision") increases
    private var loadFactor : Float {
        return Float(count) / Float(capacity)
    }
    
    private mutating func expand() { //When the load factor gets high enough, we double the capacity of the table to decrease the number of collisions
        let oldBuckets = _buckets
        _buckets = [Bucket<Key, Value>](count:oldBuckets.count * 2, repeatedValue:Bucket()) //doubling each time is a bit wasteful of memory, but dramatically decreases the amount of time spent resizing the table. You can increase it to save time but cost memory, or decrease it to save memory but cost time
        for bucket in oldBuckets {
            for (key, value) in bucket._contents {
                setWithoutExpanding(value, forKey: key) //setting everything one by one isn't fast, but we have to re-insert things to figure out where their new positions are
            }
        }
    }
    
    private mutating func setWithoutExpanding(value:Value?, forKey key:Key) {
        _buckets[bucketIndexForKey(key)].set(value, forKey: key) //Our Bucket struct takes care of handling collisions for us, so we get to just blindly look it up and set things in it :)
    }
    
    public init<Sequence:SequenceType where Sequence.Generator.Element == (Key, Value)>(pairs:Sequence) {
        _buckets = [Bucket<Key, Value>](count:max(pairs.underestimateCount(), 1), repeatedValue:Bucket()) //attempt to presize our array so we don't have to resize repeatedly. Not all SequenceType-conforming types know how long they are up front though, so we have to use underestimateCount(). For common ones like Array that will just return the count.
        for (key, value) in pairs {
            self[key] = value
        }
    }
    
    public init(dictionaryLiteral elements: (Key, Value)...) {
        self.init(pairs: elements)
    }
    
    public var count : Int {
        return _buckets.reduce(0) { (runningTotal, bucket) in
            runningTotal + bucket.count
        }
    }
    
    public subscript(key: Key) -> Value? {
        get {
            return _buckets[bucketIndexForKey(key)].get(key) //get the bucket, then search it for the key
        }
        set {
            if loadFactor > 0.75 { //0.75 is an arbitary number. You can increase it to save memory but cost time, or decrease it to save time but cost memory
                expand()
            }
            setWithoutExpanding(newValue, forKey: key)
        }
    }
    
    public var debugDescription : String {
        let contentsDesc = _buckets.map { $0.debugDescription }.joinWithSeparator(", ")
        return "MyDictionary(count:\(count), capacity:\(capacity), contents:\(contentsDesc)"
    }
    
    public func generate() -> AnyGenerator<(Key, Value)> {
        /*
        This is a little hairy, and I should probably just give in and make a little MyDictionaryGenerator struct, but bear with me.
        We need these two variables outside the closure so that we can capture them and use them to store state between invocations of the closure.
        The idea is that for each bucket, we iterate over each element in the bucket, and when we run out, we go to the next bucket and do it again. When we run out of buckets and run out of contents of the last bucket, we're done and can exit the loop and return nil
        */
        var bucketGenerator = _buckets.generate()
        var bucketContentsGenerator = bucketGenerator.next()?.generate()
        return anyGenerator({
            repeat {
                if let result = bucketContentsGenerator?.next() {
                    return result
                } else {
                    bucketContentsGenerator = bucketGenerator.next()?.generate()
                }
            } while bucketContentsGenerator != nil
            return nil
        })
    }
}

var test:MyDictionary = ["a":"b", "cow":"dog", "e":"f", "abc":"def"]
test["foo"] = "bar"
print(test)
test["a"] = "abababa"
let success = test["a"]
let failure = test["NotFound"]
test["foo"] = nil
let shouldBeNil = test["foo"]

for (key, value) in test {
    print(key, value)
}

let test2 = MyDictionary(pairs: test) //Since MyDictionary is a SequenceType of pairs, we can use it to construct another MyDictionary… of course that doesn't accomplish anything that simply assigning it wouldn't, since it's a struct, but it is neat that it works :)

let test3 = test2.map { (key, value) in (key, value.uppercaseString) } //woo protocol extensions! We get map() for free
for (key, value) in test3 {
    print(key, value)
}

/*
There's a ton of things we could improve here (beyond the obvious adding features like additional useful protocol conformances)…

The "hang a list off of each bucket" approach to dealing with hash collisions is called "separate chaining". There's an alternative scheme called "open addressing" that works by storing a single element in each bucket, and jumping to a new bucket if the first one we try is full (and jumping again if subsequent ones are). This can be significantly more efficient in some cases, since it stores everything in a single flat array. However, there's also some subtleties to it; in particular, how do you decide which bucket to jump to if there's a collision? Popular approaches include:
    • "Linear probing", where you just try the next bucket in the array. This is simple, and makes great use of cpu caches, but is vulnerable to hashValue implementations that mostly return numbers that are very close to each other, since they lead to "clusters" of full buckets with many empty useless buckets between them.
    • "Exponential/quadratic probing", where you skip an increasing number of buckets with each collision. This can avoid some of the places linear probing falls down.
    • "Double hashing", where you find the next bucket to jump to by hashing the previous hash value. This increases the overhead of probing, but can reduce the number of probes by improving distribution.
    • Hybrid schemes like "probe linearly for the size of one cpu cacheline, then do an exponential jump, then linear again, and so on".

Another set of approaches to reducing the cost of collisions is to try to reduce the number of checks (the size of N in O(N)) we have to make for some values, at the expense of slowing down other values. These include cuckoo hashing (https://en.wikipedia.org/wiki/Cuckoo_hashing) and Robin Hood hashing (http://sebastiansylvan.com/2013/05/08/robin-hood-hashing-should-be-your-default-hash-table-implementation/)

Storing the hash value alongside the key can be an easy way to reduce the cost of expand() (since you don't have to call .hashValue again for each element), and can speed up open addressing (since you can check for equal hashValues before calling the potentially slow == implementation of the key)

Some probing schemes work best if the number of buckets is prime, due to the way they "loop around" in their search. For a simple example of why, consider a simple probing scheme that hops over one bucket per check, and the following table:
    
    [empty, collision, empty, collision]
               ^
              start here

    if we start with the second bucket as shown above, then we'll incorrectly conclude that the table is full, and we have to resize. If we had used a count of 5 instead, the "every other bucket" probing scheme would search every slot before concluding it had failed to find an empty one. If you search for "prime" on http://www.opensource.apple.com/source/CF/CF-855.11/CFBasicHash.c you can see one approach to doing this.
*/
